#!/usr/bin/perl -w
# $Id: mt-to-drupal.cgi,v 1.20 2004/07/10 05:29:16 jseng Exp $

use strict;

###
# Please configure these
###

# MT blog id you want to export from
my $blog_id  = 1;          

# Import mode :-  
#   0 - Drupal 4 Blogger
#   1 - Drupal 4.4.1
#   2 - Drupal CVS
my $mode  = 0;

# your timezone -12 to +12
my $timezone = 0;

# your mysql settings
my $db_host       = "localhost";
my $db_database   = "drupal";
my $db_user       = "drupal";
my $db_pass       = "drupal";

# set this only if you use db_prefix 
# e.g. my $db_prefix = "drupal_";
my $db_prefix     = "";

# your MT RSS feed
my $rss1_0        = "index.rdf";
my $rss2_0        = "index.xml";

###
# Modify these only if you know what you are doing
###

# preserve entries and comments id? 0=no, 1=yes
# If set to 1, ie yes, this script *cannot* import into
my $preserve_id = 0;

# debug message level - 0=basic, 1=some, 2=detail
my $debug         = 1;

###
# Do not modify from here
###
my($MT_DIR);
BEGIN {
  if ($0 =~ m!(.*[/\\])!) {
    $MT_DIR = $1;
  } else {
    $MT_DIR = './';
  }
  unshift @INC, $MT_DIR . 'lib';
  unshift @INC, $MT_DIR . 'extlib';
}

use Mysql;

use MT;
use MT::Blog;
use MT::Author;
use MT::Permission;
use MT::Category;
use MT::Entry;
use MT::Comment;
use MT::Util qw( archive_file_for );
use MT::TemplateMap;
use MT::Trackback;
use MT::TBPing;
use POSIX;

local $| = 1;

sub db_title {
  print "\n--\n";
  print "-- ".$_[0]."\n";
  print "--\n\n";
}

sub db_query {
  my $dbh = shift;
  my $sql = shift;
  
  my $statement = sprintf($sql,@_);
  $dbh->Query($statement) || print "-- error! ".$Mysql::db_errstr. " - " . $statement;
  print $statement."\n" if ($debug>=2);
}

sub timeformat {
  my $mttime = shift; $mttime =~ s/\D//g;
  my($y, $mo, $d, $h, $m, $s) = unpack 'A4A2A2A2A2A2', $mttime;
  my $time = POSIX::mktime($s, $m, $h, $d, $mo - 1, $y - 1900) + $timezone;
  return $time;
}

my $dbh = Mysql->Connect($db_host, $db_database, $db_user, $db_pass);

print "Content-Type: text/html\n\n";
print "<html><pre>\n";

my $mt = MT->new(Config => $MT_DIR . 'mt.cfg',
                Directory => $MT_DIR) || die MT->errstr;

my $sth; my @return;
my $drupal_uid      = 0;
my $drupal_category = 0;
my $drupal_term     = 0;        # number of terms in drupal
my $drupal_node     = 0;        # number of nodes in drupal
my $drupal_comment  = 0;        # number of comments in drupal

$timezone = $timezone*60*60;

# find blog
my $blog = MT::Blog->load($blog_id);
if (!$blog) {
  print "-- Cannot find MT blog $blog_id!\n";
  print "</pre></html>";
  exit;
}

# find default user name
my %drupal_users;
$sth = $dbh->Query("SELECT uid, name FROM ${db_prefix}users");
while (@return = $sth->FetchRow) {
  $drupal_users{$return[1]} = $return[0];
  $drupal_uid = ($return[0] > $drupal_uid ? $return[0] : $drupal_uid);
}
if ($drupal_uid == 0) {
  print "-- You have not created your first user on drupal yet!\n";
  print "</pre></html>";
  exit;
}

$sth = $dbh->Query("SELECT vid FROM ${db_prefix}vocabulary WHERE nodes LIKE '%blog%'");
while (@return = $sth->FetchRow) {
  $drupal_category = $return[0];
  last;
}
if ($drupal_category == 0) {
  print "-- You have not created an vocabulary attached to blog yet!\n";
  print "</pre></html>";
  exit;
}

my %drupal_categories;
$sth = $dbh->Query(sprintf("SELECT name,tid FROM ${db_prefix}term_data WHERE vid=%d", $drupal_category));
while (@return = $sth->FetchRow) {
  $drupal_categories{$return[0]} = $return[1];
}

$sth = $dbh->Query("SELECT name, id FROM ${db_prefix}sequences");
while (@return = $sth->FetchRow) {
  $drupal_term = $return[1] if ($return[0] =~ "${db_prefix}term_data_tid");
  $drupal_node = $return[1] if ($return[0] =~ "${db_prefix}node_nid");
  $drupal_comment = $return[1] if ($return[0] =~ "${db_prefix}comments_cid");
}

my $iter; my $count; my @userid; my @catsid;

db_title('Creating additional bloggers');
my @authors = MT::Author->load(); $count = $drupal_uid;
for my $author (@authors) {
  my $perms = MT::Permission->load({author_id => $author->id, 
                                    blog_id   => $blog_id});
  next if ($perms && !$perms->can_post);

  if ($drupal_users{$author->name}) {
    $userid[$author->id] = $drupal_users{$author->name};
    next;
  }

  $count++;
  $userid[$author->id] = $count;

  if ($mode == 0 || $mode == 1) {
    db_query($dbh,"INSERT INTO ${db_prefix}users (uid, rid, name, pass, mail, timestamp) VALUES (%d, %d, %s, %s, %s, %d)", $count, 2, $dbh->quote($author->name), $dbh->quote($author->password), $dbh->quote($author->email), time());
  } else {
    db_query($dbh,"INSERT INTO ${db_prefix}users (uid, name, pass, mail, created) VALUES (%d, %s, %s, %s, %d)", $count, $dbh->quote($author->name), $dbh->quote($author->password), $dbh->quote($author->email), time());
  }
}
db_query($dbh,"REPLACE INTO ${db_prefix}sequences VALUES (%s, %d)", $dbh->quote("${db_prefix}users_uid"), $count);
db_title(sprintf("%s bloggers imported",$count - $drupal_uid));

db_title("Creating categories");
my @categories = MT::Category->load({blog_id => $blog_id}); $count = $drupal_term;
for my $cat (@categories) {
  my $path = $cat->label;
  $path =~ tr/A-Z/a-z/;
  $path =~ s/\s/_/g;
  $path = "archives/cat_".$path.".html";
  if ($drupal_categories{$cat->label}) {
    $catsid[$cat->id] = $drupal_categories{$cat->label};
    db_query($dbh, "REPLACE INTO ${db_prefix}url_alias (src, dst) VALUES (%s, %s)", $dbh->quote(sprintf("taxonomy/page/or/%d",$catsid[$cat->id])),$dbh->quote($path));
    next;
  }
  $count++;
  $catsid[$cat->id] = $count;
  db_query($dbh, "REPLACE INTO ${db_prefix}url_alias (src, dst) VALUES (%s, %s)", $dbh->quote(sprintf("taxonomy/page/or/%d",$count)),$dbh->quote($path));
  db_query($dbh, "INSERT INTO ${db_prefix}term_data (vid, tid, name, description) VALUES (%d, %d, %s, %s)", $drupal_category, $count, $dbh->quote($cat->label), $dbh->quote($cat->description));
  db_query($dbh, "INSERT INTO ${db_prefix}term_hierarchy (tid, parent) VALUES (%d, %d)", $count, 0);

  if ($cat->allow_pings) {
    # unable to ping to taxonomy's terms yet.
  }
}
db_query($dbh, "REPLACE INTO ${db_prefix}sequences VALUES (%s, %d)", $dbh->quote("${db_prefix}term_data_tid"), $count);
db_title(sprintf("%s categories imported",$count - $drupal_term));

db_title("Creating entries");
my $total_entries = MT::Entry->count({blog_id => $blog_id});
my $cid = $drupal_comment; $count = $drupal_node;
my $lnid = 0;
for (my $offset = 0; $offset < $total_entries; $offset += 50) {
  db_title(sprintf("importing entry %d to %d",$offset,($offset+50>$total_entries ? $total_entries : $offset+50))) if ($debug>=1);
  my @entries = MT::Entry->load({blog_id => $blog_id}, {limit => 50, offset => $offset});
  for my $entry (@entries) {
    $count++; $lnid = $entry->id if ($entry->id > $lnid);
    my $entry_status = 1;
    $entry_status = 0 if ($entry->status != 2);
    if ($mode == 0) {
      db_query($dbh,"INSERT INTO ${db_prefix}node (nid, type, title, uid, status, created, changed, comment, promote, body, teaser) VALUES (%d, %s, %s, %d, %d, %d, %d, %d, %d, %s, %s)", ($preserve_id ? $entry->id : $count), $dbh->quote('blog'), $dbh->quote($entry->title), ($userid[$entry->author_id] ? $userid[$entry->author_id] : 1), $entry_status, timeformat($entry->created_on), timeformat($entry->modified_on), ($entry->allow_comments ? 2 : 0), 1, $dbh->quote(($entry->text ? $entry->text : '')), $dbh->quote(($entry->text ? $entry->text : '')));
      db_query($dbh,"INSERT INTO ${db_prefix}blog (nid, extended, excerpt, format) VALUES (%d, %s, %s, %d)", ($preserve_id ? $entry->id : $count), $dbh->quote(($entry->text_more ? $entry->text_more: '')), $dbh->quote(($entry->excerpt ? $entry->excerpt : '')), ($entry->convert_breaks ? 0 : -1));
      db_query($dbh,"INSERT INTO ${db_prefix}trackback (nid, toping, pinged) VALUES (%d, %s, %s)", ($preserve_id ? $entry->id : $count), $dbh->quote($entry->to_ping_urls), $dbh->quote($entry->pinged_urls));
    } else {
      my $body = '';
      if ($entry->text) {
        $body = $entry->text;
        $body .= "<!--break-->" . $entry->text_more if ($entry->text_more);
      }
      db_query($dbh,"INSERT INTO ${db_prefix}node (nid, type, title, uid, status, created, changed, comment, promote, body, teaser) VALUES (%d, %s, %s, %d, %d, %d, %d, %d, %d, %s, %s)", ($preserve_id ? $entry->id : $count), $dbh->quote('blog'), $dbh->quote($entry->title), ($userid[$entry->author_id] ? $userid[$entry->author_id] : 1), $entry_status, timeformat($entry->created_on), timeformat($entry->modified_on), ($entry->allow_comments ? 2 : 0), 1, $dbh->quote($body), $dbh->quote(($entry->text ? $entry->text : '')));
    }
  
    my $cats = $entry->categories;
    for my $cat (@$cats) {
      next if (!$catsid[$cat->id]);
      db_query($dbh,"INSERT INTO ${db_prefix}term_node (nid, tid) VALUES (%d, %d)", ($preserve_id ? $entry->id : $count), $catsid[$cat->id]);
    }
  
    my @comments = MT::Comment->load({blog_id => $blog_id, entry_id => $entry->id}); 
    my $ccid = 0; my $thread;
    for my $comment (@comments) {
      $cid++; $ccid++; $thread = "";
      for (my $i = 0; $i < int $ccid/10; $i++) {
        $thread .= "9";
      }
      $thread .= "" . $ccid%10 . "/";
#    db_query($dbh, "UPDATE ${db_prefix}comments SET thread = %s WHERE cid = %d", $dbh->quote($thread), $comment[0]);
      if ($mode == 0 || $mode == 2) {
        db_query($dbh,"INSERT INTO ${db_prefix}comments (cid, pid, nid, uid, subject, comment, hostname, timestamp, status, name, mail, homepage, thread) VALUES (%d, %d, %d, %d, %s, %s, %s, %d, %d, %s, %s, %s, %s)", $cid, 0, ($preserve_id ? $entry->id : $count), 0, $dbh->quote(''), $dbh->quote($comment->text), $dbh->quote($comment->ip), timeformat($comment->created_on), 0, $dbh->quote($comment->author), $dbh->quote($comment->email), $dbh->quote($comment->url), $dbh->quote($thread));
      } else {
        db_query($dbh,"INSERT INTO ${db_prefix}comments (cid, pid, nid, uid, subject, comment, hostname, timestamp, status, thread) VALUES (%d, %d, %d, %d, %s, %s, %s, %d, %d, %s)", $cid, 0, ($preserve_id ? $entry->id : $count), 0, $dbh->quote(''), $dbh->quote($comment->text), $dbh->quote($comment->ip), timeformat($comment->created_on), 0, $dbh->quote($thread));
      }
    }
  
    my $tbping = MT::Trackback->load({blog_id => $blog_id, entry_id => $entry->id}); 
    if ($tbping) {
      my @trackbacks = MT::TBPing->load({blog_id => $blog_id, tb_id => $tbping->id});
      for my $tb (@trackbacks) {
        $cid++; $ccid++; $thread = "";
        for (my $i = 0; $i < int $ccid/10; $i++) {
          $ thread .= "9";
        }
        $thread .= "" . $ccid%10 . "/";
        my $text = sprintf("<strong>Trackback from <a href=\"%s\">%s</a>:</strong><br /><blockquote>%s...</blockquote>", $tb->source_url, $tb->blog_name, $tb->excerpt);
        if ($mode == 0 || $mode == 2) {
          db_query($dbh,"INSERT INTO ${db_prefix}comments (cid, pid, nid, uid, subject, comment, hostname, timestamp, status, name, mail, homepage, thread) VALUES (%d, %d, %d, %d, %s, %s, %s, %d, %d, %s, %s, %s, %s)", $cid, ($preserve_id ? $entry->id : $count), ($preserve_id ? $entry->id : $count), 0, $dbh->quote(($tb->title ? $tb->title : '')), $dbh->quote($text), $dbh->quote($tb->ip), timeformat($tb->created_on), 0, $dbh->quote(($tb->blog_name ? $tb->blog_name : '')), $dbh->quote(''), $dbh->quote($tb->source_url), $dbh->quote($thread));
        } else {
          db_query($dbh,"INSERT INTO ${db_prefix}comments (cid, pid, nid, uid, subject, comment, hostname, timestamp, status, thread) VALUES (%d, %d, %d, %d, %s, %s, %s, %d, %d, %s)", ($preserve_id ? $entry->id : $count), $cid, ($preserve_id ? $entry->id :$count), 0, $dbh->quote(($tb->title ? $tb->title : '')), $dbh->quote($text), $dbh->quote($tb->ip), timeformat($tb->created_on), 0, $dbh->quote($thread));
        }
        db_query($dbh,"REPLACE INTO ${db_prefix}trackback_pingme (nid, cid) VALUES (%d, %d)", ($preserve_id ? $entry->id : $count), $cid) if ($mode == 0);
      }
    }
  
    my $file = $blog->archive_url . sprintf("%06d", $entry->id) . ".html";
    my $site_url = $blog->site_url;
    $file =~ s/$site_url//;
    db_query($dbh,"REPLACE INTO ${db_prefix}url_alias (src, dst) VALUES (%s, %s)", $dbh->quote(sprintf("node/view/%d", ($preserve_id ? $entry->id : $count))), $dbh->quote($file));
  }
}
db_query($dbh, "REPLACE INTO ${db_prefix}sequences VALUES (%s, %d)", $dbh->quote("${db_prefix}comments_cid"), $cid);
db_query($dbh, "REPLACE INTO ${db_prefix}sequences VALUES (%s, %d)", $dbh->quote("${db_prefix}node_nid"), ($preserve_id ? $lnid : $count));
db_title(sprintf("%s entries imported",$count - $drupal_node));
db_title(sprintf("%s comments/trackbacks imported",$cid - $drupal_comment));

db_title("Alias'ing MT RSS feed to Drupal RSS feed");
db_query($dbh, "REPLACE INTO ${db_prefix}url_alias (src, dst) VALUES (%s, %s)", $dbh->quote("node/feed/rss1"), $dbh->quote($rss1_0));
db_query($dbh, "REPLACE INTO ${db_prefix}url_alias (src, dst) VALUES (%s, %s)", $dbh->quote("node/feed/rss2"), $dbh->quote($rss2_0));

db_title("Autosubscribe all users to their own post");
db_query($dbh, "INSERT INTO ${db_prefix}subscriptions (sid, uid, stype) SELECT nid, uid, 'node' FROM ${db_prefix}node");

print "\n</pre></html>\n\n";
